#!/usr/bin/env python
"""
program:    nitrogen-chwall.py
version:    1.1
author:     James Hendrie (hendrie dot james at gmail dot com)
license:    GPLv3
date:       2013-01-29

A quick and dirty script to use the Nitrogen program to change the background
image (wallpaper) on any desktop it controls.  How it works:  Alter the bg-saved
config file so that the selected image is the 'saved' one, and then call
nitrogen --restore.  Pretty simple.
"""

import os, sys, shutil

#   The default path of the config file.  I'm putting this up here in case
#   it gets changed in future versions of Nitrogen
cfgPath = os.path.expanduser("~/.config/nitrogen")
cfgFile = cfgPath + "/bg-saved.cfg"

#   Whether or not you want to backup the cfg file.  True = yes, False = no.
bkp = True
#   Backup file location
bkpFile = cfgFile + ".bkp"

#   The system call that we use to do the actual desktop changing
systemCall = "nitrogen --restore"

def print_help():
    """
    Print a help screen
    """
    print("\nUsage:  nitrogen-chwall IMAGE\n")
    print("This script modifies the bg-saved.cfg file used by Nitrogen and")
    print("then calls 'nitrogen --restore' to quickly change the wallpaper to")
    print("the selected image.  This is a script for the lazy, or those who")
    print("make use of Thunar's custom actions to any extent.\n")
    print("There are two options:")
    print("     -h or --help:       Print this screen.")
    print("     -v or --version:    Version and Author info.")
    print("     -r or --revert:     Restore config backup (revert)\n")
    print("This should go without saying, but the 'nitrogen' program is quite")
    print("necessary for this script to function properly.\n")

def print_version():
    """
    Print version and author info
    """
    print("\nProgram:   nitrogen-chwall.py version 1.1")
    print("Author:    James Hendrie (hendrie dot james at gmail dot com)\n")

def read_cfg():
    """
    Open up the config file (according to variable cfgFile), read the lines from
    it, close the file and return the text to main.
    """

    #   Try to open the cfg file.  If it fails, print an error and exit.
    try:
        bgSavedFile = open(cfgFile, "r")        #   Open file
        bgSavedText = bgSavedFile.readlines()   #   Read lines to list
        bgSavedFile.close()                     #   Close file
        return bgSavedText                      #   Return text

    except FileNotFoundError or OSError or PermissionError:
        print("Error:  Can't open the bg-saved.cfg file for reading")
        sys.exit()

def write_cfg(bgSavedText, imageName):
    """
    Get bgSavedText and imageName from main, go through the file and modify the
    appropriate line, then write all of the (possibly modified lines) back to 
    recently opened file 'bgSavedFile'.
    """

    #   If bkp is true, copy the config file to the backup location
    if bkp == True:
        shutil.copy(cfgFile, bkpFile)

    #   Open the config file again
    bgSavedFile = open(cfgFile, "w")

    #   For each line, if it's the line we're looking for, modify said line
    #   to include the new path to the previously chosen image.  We write all
    #   lines to the opened cfg file, even the unmodified ones (the old config
    #   file is overwritten)
    for l in bgSavedText:
        if "file=" in l:
            l = ""          #   Blank out the line
            l += "file="    #   Add this necessary part of the line back
            l += imageName  #   Add full path to the image to the line

        #   If the line doesn't have a newline at the end of it, add one
        if l[-1] != '\n':
            l += '\n'

        #   Write to the file
        bgSavedFile.write(l)

    #   Close the file, 'saving' it
    bgSavedFile.close()

def revert():
    """
    This function restores the backup file
    """
    #   Copy the backup file to the config file location
    shutil.copy(bkpFile, cfgFile)

    #   Call nitrogen
    os.system(systemCall)



def main():
    #   ARGUMENT HANDLING
    #
    #   If there are more than two arguments (one not including the script),
    #   then tell the user that there are too many and exit.
    if ( len(sys.argv) > 2):
        print("\nError:  Too many arguments.")
        print("Usage:  nitrogen-chwall IMAGE_FILENAME\n")
        sys.exit()
    elif ( len(sys.argv) < 2):
        print("\nError:  Too few arguments.")
        print("Usage:  nitrogen-chwall IMAGE_FILENAME\n")
        sys.exit()

    #   If the user tries to access the help, print the help screen
    if ( (sys.argv[-1] == "-h") or (sys.argv[-1] == "--help")):
        print_help()
        sys.exit()

    # If the user tries to get version info, plague him with author info as well
    if ( (sys.argv[-1] == "-v") or (sys.argv[-1] == "--version") ):
        print_version()
        sys.exit()

    #   If the user wants to restore the backup
    if ( (sys.argv[-1] == "-r") or (sys.argv[-1] == "--revert") ):
        revert()
        sys.exit()

    #   Full path to the image (should be the only argument passed to script)
    imageName = os.path.abspath(sys.argv[-1])

    #   Get the text from the read config file
    bgSavedText = read_cfg()

    #   Overwrite said file with modified text
    write_cfg(bgSavedText, imageName)

    #   Finally, call the program and have it 'restore' the cfg file
    os.system(systemCall)


if __name__ == "__main__":
    main()
