#!/bin/bash
################################################################################
#   nitrogen-chwall.sh  | Version 1.1 | GPLv3 | 2013.01.29
#   James Hendrie       |   hendrie dot james at gmail dot com
################################################################################

##  The config directory
cfgDir="$HOME/.config/nitrogen"

##  Config file name
cfgFile="bg-saved.cfg"

##  The config file used by nitrogen to remember the selected wallpaper.  This
##  is currently found at ~/.config/nitrogen/bg-saved.cfg, though it might
##  change in the future.
cfg="$cfgDir/$cfgFile"

##  Option determining whether or not to back up the config file.  Default to ON
bkp=1
bkpCfg="$cfg.bkp"

##  The directory in which we'll write the temporary file.  We'll need to test
##  to see if we can write to it.
tempDir="/tmp"

##  The temporary file to which we'll write
tempFile="$tempDir/tmp-$cfgFile"


function revert
{
    if [ ! -e $bkpCfg ]; then
        echo "Error:  Backup file ($bkpCfg) does not exist" 1>&2
        exit
    fi

    if [ ! -w $cfg ]; then
        echo "Error:  Config file ($cfg) cannot be written to" 1>&2
        exit
    fi

    cp $bkpCfg $cfg
    nitrogen --restore
}

##  Test to make sure the config directory exists and can be written to
if [ ! -e $cfgDir ]; then
    echo "Error:  Nitrogen config directory ($cfgDir) does not exist" 1>&2
    exit
else
    if [ ! -w $cfgDir ]; then
        echo "Error:  Cannot write to config directory ($cfgDir)" 1>&2
        exit
    fi
fi

##  Check for the existence of the config file
if [ ! -e $cfg ]; then
    echo "Error:  $cfg does not exist." 1>&2
    exit
fi

##  Check to make sure we can write to the temporary directory
if [ ! -w $tempDir ]; then
    echo "Error:  Cannot write to $tempDir" 1>&2
    exit
fi

##  Check the number of arguments, tell the user what's up if he gets it wrong.
if [ $# -lt 1 ]; then
    echo "Error:  Too few arguments." 1>&2
    echo "Usage:  nitrogen-chwall IMAGE_NAME" 1>&2
    exit
elif [ $# -gt 1 ]; then
    echo "Error:  Too many arguments." 1>&2
    echo "Usage:  nitrogen-chwall IMAGE_NAME" 1>&2
    exit
fi

##  Check for the three arguments accepted
if [ $1 == '-h' ] || [ $1 == '--help' ]; then
    echo -e "Usage:  nitrogen-chwall.sh IMAGE_NAME"
    echo -e "\nOptions:"
    echo -e "   -h or --help:       Print this help screen"
    echo -e "   -v or --version:    Print version and author info"
    echo -e "   -r or --revert:     Restore the config backup (revert)"
    exit
elif [ $1 == '--version' ] || [ $1 == '-v' ]; then
    echo -e "nitrogen-chwall.sh, version 1.1, licensed under the GPLv3"
    echo -e "Written by James Hendrie (hendrie dot james at gmail dot com)"
    exit
elif [ $1 == '--revert' ] || [ $1 == '-r' ]; then
    revert
    exit
fi

##  Check to make sure the passed argument (file) exists, and can be written to
if [ ! -e $1 ]; then
    echo "Error:  File does not exist ($1)" 1>&2
    exit
else
    if [ ! -w $1 ]; then
        echo "Error:  Cannot write to file ($1)" 1>&2
        exit
    fi
fi

##  The full path of the image you're passing as an argument.
imageName=$(readlink -f $1)

##  Cat the file, and for every line read in, check and see if it's the one that
##  we're looking for.  If it is, create a new line with the given image file
##  and append that into the temp file in stead of the original line.  Otherwise
##  just append the old lines to the temp file.

cat $cfg | while read LINE
do
    checkMe=$(echo $LINE | head -c 5)
    if [ $checkMe == 'file=' ]; then
        echo "file=$imageName" >> $tempFile
    else
        echo $LINE >> $tempFile
    fi
done

##  If bkp is 1, copy the original config to the bkpCfg location
if [ $bkp == 1 ]; then
    cp $cfg $bkpCfg
fi

##  Move the temp file to the original config file, overwriting it
mv $tempFile $cfg

##  Finally, call the program (nitrogen) with the --restore argument.
nitrogen --restore
